import json
import logging
import sys

import azure.functions as func
from azure.eventgrid import EventGridEvent


def main(event: func.EventGridEvent):
    logging.info(sys.version)
    logging.info(event)
    result = json.dumps(
        {
            "id": event.id,
            "data": event.get_json(),
            "topic": event.topic,
            "subject": event.subject,
            "event_type": event.event_type,
        }
    )
    logging.info(result)
    deserialized_event = EventGridEvent.from_dict(json.loads(result))
    ## can only be EventGridEvent
    print("event: {}".format(event))



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
FILE: sample_consume_process_events_async.py
DESCRIPTION:
    These samples demonstrate sending, receiving, releasing, and acknowledging CloudEvents.
USAGE:
    python sample_consume_process_events_async.py
    Set the environment variables with your own values before running the sample:
    1) EVENTGRID_KEY - The access key of your eventgrid account.
    2) EVENTGRID_ENDPOINT - The namespace endpoint. Typically it exists in the format
    "https://<YOUR-NAMESPACE-NAME>.<REGION-NAME>.eventgrid.azure.net".
    3) EVENTGRID_TOPIC_NAME - The namespace topic name.
    4) EVENTGRID_EVENT_SUBSCRIPTION_NAME - The event subscription name.
"""
import os
import asyncio
from azure.core.credentials import AzureKeyCredential
from azure.eventgrid.models import *
from azure.core.messaging import CloudEvent
from azure.core.exceptions import HttpResponseError
from azure.eventgrid.aio import EventGridConsumerClient, EventGridPublisherClient

EVENTGRID_KEY: str = os.environ["EVENTGRID_KEY"]
EVENTGRID_ENDPOINT: str = os.environ["EVENTGRID_ENDPOINT"]
TOPIC_NAME: str = os.environ["EVENTGRID_TOPIC_NAME"]
EVENT_SUBSCRIPTION_NAME: str = os.environ["EVENTGRID_EVENT_SUBSCRIPTION_NAME"]


async def run():
    # Create a client
    publisher = EventGridPublisherClient(
        EVENTGRID_ENDPOINT, AzureKeyCredential(EVENTGRID_KEY), namespace_topic=TOPIC_NAME
    )
    client = EventGridConsumerClient(
        EVENTGRID_ENDPOINT,
        AzureKeyCredential(EVENTGRID_KEY),
        namespace_topic=TOPIC_NAME,
        subscription=EVENT_SUBSCRIPTION_NAME,
    )

    cloud_event_reject = CloudEvent(data="reject", source="https://example.com", type="example")
    cloud_event_release = CloudEvent(data="release", source="https://example.com", type="example")
    cloud_event_ack = CloudEvent(data="acknowledge", source="https://example.com", type="example")
    cloud_event_renew = CloudEvent(data="renew", source="https://example.com", type="example")

    async with publisher, client:
        # Send Cloud Events
        await publisher.send(
            [
                cloud_event_reject,
                cloud_event_release,
                cloud_event_ack,
                cloud_event_renew,
            ]
        )

        # Receive Published Cloud Events
        try:
            receive_results = await client.receive(
                max_events=10,
                max_wait_time=10,
            )
        except HttpResponseError:
            raise

        # Iterate through the results and collect the lock tokens for events we want to release/acknowledge/reject/renew:

        release_events = []
        acknowledge_events = []
        reject_events = []
        renew_events = []

        for detail in receive_results:
            data = detail.event.data
            broker_properties = detail.broker_properties
            if data == "release":
                release_events.append(broker_properties.lock_token)
            elif data == "acknowledge":
                acknowledge_events.append(broker_properties.lock_token)
            elif data == "renew":
                renew_events.append(broker_properties.lock_token)
            else:
                reject_events.append(broker_properties.lock_token)

        # Release/Acknowledge/Reject/Renew events

        if len(release_events) > 0:
            try:
                release_result = await client.release(
                    lock_tokens=release_events,
                )
            except HttpResponseError:
                raise

            for succeeded_lock_token in release_result.succeeded_lock_tokens:
                print(f"Succeeded Lock Token:{succeeded_lock_token}")

        if len(acknowledge_events) > 0:
            try:
                ack_result = await client.acknowledge(
                    lock_tokens=acknowledge_events,
                )
            except HttpResponseError:
                raise

            for succeeded_lock_token in ack_result.succeeded_lock_tokens:
                print(f"Succeeded Lock Token:{succeeded_lock_token}")

        if len(reject_events) > 0:
            try:
                reject_result = await client.reject(
                    lock_tokens=reject_events,
                )
            except HttpResponseError:
                raise

            for succeeded_lock_token in reject_result.succeeded_lock_tokens:
                print(f"Succeeded Lock Token:{succeeded_lock_token}")

        if len(renew_events) > 0:
            try:
                renew_result = await client.renew_locks(
                    lock_tokens=renew_events,
                )
            except HttpResponseError:
                raise

            for succeeded_lock_token in renew_result.succeeded_lock_tokens:
                print(f"Succeeded Lock Token:{succeeded_lock_token}")


if __name__ == "__main__":
    asyncio.run(run())



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import logging
import sys
import os
import json
import pytest
import uuid
from msrest.serialization import UTC
import datetime as dt

from devtools_testutils import AzureRecordedTestCase
from azure.core.messaging import CloudEvent
from azure.core.credentials import AzureKeyCredential
from azure.eventgrid import EventGridConsumerClient, EventGridPublisherClient
from eventgrid_preparer import (
    EventGridPreparer,
)


class TestEventGridConsumerClient(AzureRecordedTestCase):
    def create_eg_publisher_client(self, endpoint, topic=None):
        credential = self.get_credential(EventGridPublisherClient)
        client = self.create_client_from_credential(
            EventGridPublisherClient, credential=credential, endpoint=endpoint, namespace_topic=topic
        )
        return client

    def create_eg_consumer_client(self, endpoint, topic, subscription):
        credential = self.get_credential(EventGridConsumerClient)
        client = self.create_client_from_credential(
            EventGridConsumerClient,
            credential=credential,
            endpoint=endpoint,
            namespace_topic=topic,
            subscription=subscription,
        )
        return client

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_receive_data(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_key = kwargs["eventgrid_key"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        eventgrid_event_subscription_name = kwargs["eventgrid_event_subscription_name"]
        publisher = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        consumer = self.create_eg_consumer_client(
            eventgrid_endpoint, eventgrid_topic_name, eventgrid_event_subscription_name
        )
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )
        publisher.send(cloud_event)

        received_event = consumer.receive(max_events=1)
        assert len(received_event) == 1

        for event in received_event:
            ack_result = consumer.acknowledge(lock_tokens=[event.broker_properties.lock_token])
            assert ack_result.succeeded_lock_tokens == [event.broker_properties.lock_token]

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_receive_renew_data(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_key = kwargs["eventgrid_key"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        eventgrid_event_subscription_name = kwargs["eventgrid_event_subscription_name"]
        publisher = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        consumer = self.create_eg_consumer_client(
            eventgrid_endpoint, eventgrid_topic_name, eventgrid_event_subscription_name
        )

        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )
        publisher.send(cloud_event)

        received_event = consumer.receive(max_events=1)
        assert len(received_event) == 1

        for event in received_event:
            renew_lock = consumer.renew_locks(lock_tokens=[event.broker_properties.lock_token])
            ack_result = consumer.acknowledge(lock_tokens=[event.broker_properties.lock_token])
            assert ack_result.succeeded_lock_tokens == [event.broker_properties.lock_token]

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_receive_release_data(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_key = kwargs["eventgrid_key"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        eventgrid_event_subscription_name = kwargs["eventgrid_event_subscription_name"]
        publisher = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        consumer = self.create_eg_consumer_client(
            eventgrid_endpoint, eventgrid_topic_name, eventgrid_event_subscription_name
        )

        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )
        publisher.send(cloud_event)

        received_event = consumer.receive(max_events=1)
        assert len(received_event) == 1

        for event in received_event:
            release = consumer.release(lock_tokens=[event.broker_properties.lock_token])
            assert release.succeeded_lock_tokens == [event.broker_properties.lock_token]

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_receive_reject_data(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_key = kwargs["eventgrid_key"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        eventgrid_event_subscription_name = kwargs["eventgrid_event_subscription_name"]
        publisher = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        consumer = self.create_eg_consumer_client(
            eventgrid_endpoint, eventgrid_topic_name, eventgrid_event_subscription_name
        )

        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )
        publisher.send(cloud_event)

        received_event = consumer.receive(max_events=1)
        assert len(received_event) == 1

        for event in received_event:
            reject = consumer.reject(lock_tokens=[event.broker_properties.lock_token])
            assert reject.succeeded_lock_tokens == [event.broker_properties.lock_token]



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
FILE: sample_namespace_authentication.py
DESCRIPTION:
    These samples demonstrate authenticating an EventGridPublisherClient and an EventGridConsumerClient.
USAGE:
    python sample_namespace_authentication.py
    Set the environment variables with your own values before running the sample:
    1) EVENTGRID_KEY - The access key of your eventgrid account.
    2) EVENTGRID_ENDPOINT - The namespace hostname. Typically it exists in the format
    "https://<YOUR-TOPIC-NAME>.<REGION-NAME>.eventgrid.azure.net".
    3) EVENTGRID_TOPIC_NAME - The namespace topic name.
    4) EVENTGRID_EVENT_SUBSCRIPTION_NAME = The event subscription name.
"""
import os
from azure.eventgrid import EventGridPublisherClient, EventGridConsumerClient
from azure.core.credentials import AzureKeyCredential

topic_key = os.environ["EVENTGRID_KEY"]
endpoint = os.environ["EVENTGRID_ENDPOINT"]
topic_name = os.environ["EVENTGRID_TOPIC_NAME"]
event_subscription_name = os.environ["EVENTGRID_EVENT_SUBSCRIPTION_NAME"]

credential_key = AzureKeyCredential(topic_key)
publisher_client = EventGridPublisherClient(endpoint, credential_key, namespace_topic=topic_name)
consumer_client = EventGridConsumerClient(
    endpoint, credential_key, namespace_topic=topic_name, subscription=event_subscription_name
)


from azure.identity import DefaultAzureCredential
from azure.eventgrid import EventGridPublisherClient

default_az_credential = DefaultAzureCredential()
endpoint = os.environ["EVENTGRID_TOPIC_ENDPOINT"]
publisher_client = EventGridPublisherClient(endpoint, default_az_credential, namespace_topic=topic_name)
consumer_client = EventGridConsumerClient(
    endpoint, default_az_credential, namespace_topic=topic_name, subscription=event_subscription_name
)



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import os
import json
import pytest
from azure.core.exceptions import (
    HttpResponseError,
    ClientAuthenticationError,
)

from devtools_testutils import AzureMgmtRecordedTestCase, recorded_by_proxy

from azure.core.credentials import AzureKeyCredential
from azure.eventgrid import EventGridPublisherClient, EventGridEvent

from eventgrid_preparer import (
    EventGridPreparer,
)


class TestEventGridPublisherClientExceptions(AzureMgmtRecordedTestCase):
    def create_eg_publisher_client(self, endpoint):
        credential = self.get_credential(EventGridPublisherClient)
        client = self.create_client_from_credential(EventGridPublisherClient, credential=credential, endpoint=endpoint)
        return client

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_raise_on_auth_error(self, **kwargs):
        eventgrid_topic_endpoint = kwargs.pop("eventgrid_topic_endpoint")
        akc_credential = AzureKeyCredential("bad credential")
        client = EventGridPublisherClient(eventgrid_topic_endpoint, akc_credential)
        eg_event = EventGridEvent(
            subject="sample",
            data={"sample": "eventgridevent"},
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        with pytest.raises(
            ClientAuthenticationError,
            match="The request authorization key is not authorized for*",
        ):
            client.send(eg_event)

    @pytest.mark.skip("Fix during MQ - skip to unblock pipeline")
    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_raise_on_bad_resource(self, **kwargs):
        eventgrid_topic_key = kwargs.pop("eventgrid_topic_key")
        akc_credential = AzureKeyCredential(eventgrid_topic_key)
        client = EventGridPublisherClient(
            "https://bad-resource.westus-1.eventgrid.azure.net/api/events", akc_credential
        )
        eg_event = EventGridEvent(
            subject="sample", data={"sample": "eventgridevent"}, event_type="Sample.EventGrid.Event", data_version="2.0"
        )
        with pytest.raises(HttpResponseError):
            client.send(eg_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_raise_on_large_payload(self, eventgrid_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_topic_endpoint)

        path = os.path.abspath(os.path.join(os.path.abspath(__file__), "..", "./large_data.json"))
        with open(path) as json_file:
            data = json.load(json_file)
        eg_event = EventGridEvent(
            subject="sample",
            data=data,
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        with pytest.raises(HttpResponseError) as err:
            client.send(eg_event)
        assert "The maximum size (1536000) has been exceeded." in err.value.message



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------
import os
import pytest
from urllib.parse import urlparse
from devtools_testutils import (
    add_general_string_sanitizer,
    test_proxy,
    add_body_key_sanitizer,
    add_header_regex_sanitizer,
    add_oauth_response_sanitizer,
    add_body_regex_sanitizer,
)
from devtools_testutils.sanitizers import (
    add_remove_header_sanitizer,
    add_general_regex_sanitizer,
    set_custom_default_matcher,
)


@pytest.fixture(scope="session", autouse=True)
def add_sanitizers(test_proxy):
    # this can be reverted to set_bodiless_matcher() after tests are re-recorded and don't contain these headers
    set_custom_default_matcher(
        compare_bodies=False,
        excluded_headers="Authorization,Content-Length,x-ms-client-request-id,x-ms-request-id",
    )
    add_remove_header_sanitizer(headers="aeg-sas-key, aeg-sas-token, aeg-channel-name")
    add_general_regex_sanitizer(
        value="fakeresource", regex="(?<=\\/\\/)[a-z-]+(?=\\.eastus-1\\.eventgrid\\.azure\\.net/api/events)"
    )

    add_oauth_response_sanitizer()
    add_header_regex_sanitizer(key="Set-Cookie", value="[set-cookie;]")
    add_header_regex_sanitizer(key="Cookie", value="cookie;")

    add_body_key_sanitizer(json_path="$..id", value="id")

    client_id = os.getenv("AZURE_CLIENT_ID", "sanitized")
    client_secret = os.getenv("AZURE_CLIENT_SECRET", "sanitized")
    eventgrid_client_id = os.getenv("EVENTGRID_CLIENT_ID", "sanitized")
    eventgrid_client_secret = os.getenv("EVENTGRID_CLIENT_SECRET", "sanitized")
    tenant_id = os.getenv("AZURE_TENANT_ID", "sanitized")
    eventgrid_topic_endpoint = os.getenv("EVENTGRID_TOPIC_ENDPOINT", "sanitized")

    eventgrid_endpoint = os.getenv("EVENTGRID_ENDPOINT", "sanitized")
    eventgrid_key = os.getenv("EVENTGRID_KEY", "sanitized")
    eventgrid_topic_name = os.getenv("EVENTGRID_TOPIC_NAME", "sanitized")
    eventgrid_event_subscription_name = os.getenv("EVENTGRID_EVENT_SUBSCRIPTION_NAME", "sanitized")

    eventgrid_cloud_event_topic_key = os.getenv("EVENTGRID_CLOUD_EVENT_TOPIC_KEY", "sanitized")
    eventgrid_cloud_event_topic_endpoint = os.getenv("EVENTGRID_CLOUD_EVENT_TOPIC_ENDPOINT", "sanitized")

    eventgrid_topic_key = os.getenv("EVENTGRID_TOPIC_KEY", "sanitized")
    eventgrid_topic_endpoint = os.getenv("EVENTGRID_TOPIC_ENDPOINT", "sanitized")

    eventgrid_partner_channel_name = os.getenv("EVENTGRID_PARTNER_CHANNEL_NAME", "sanitized")
    eventgrid_partner_namespace_topic_endpoint = os.getenv("EVENTGRID_PARTNER_NAMESPACE_TOPIC_ENDPOINT", "sanitized")
    eventgrid_partner_namespace_topic_key = os.getenv("EVENTGRID_PARTNER_NAMESPACE_TOPIC_KEY", "sanitized")

    # Need to santize namespace for eventgrid_topic:
    try:
        eventgrid_hostname = urlparse(eventgrid_topic_endpoint).hostname
        add_general_string_sanitizer(target=eventgrid_hostname.upper(), value="sanitized.eastus-1.eventgrid.azure.net")
    except:
        pass
    add_general_string_sanitizer(target=client_id, value="00000000-0000-0000-0000-000000000000")
    add_general_string_sanitizer(target=client_secret, value="sanitized")
    add_general_string_sanitizer(target=eventgrid_client_id, value="00000000-0000-0000-0000-000000000000")
    add_general_string_sanitizer(target=eventgrid_client_secret, value="sanitized")
    add_general_string_sanitizer(target=tenant_id, value="00000000-0000-0000-0000-000000000000")



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
from typing import List, overload, Mapping, Any, Generic, TypeVar
from azure.core.messaging import CloudEvent
from ._models import (
    ReceiveDetails as InternalReceiveDetails,
    BrokerProperties as InternalBrokerProperties,
)

DataType = TypeVar("DataType")


class ReceiveDetails(InternalReceiveDetails, Generic[DataType]):
    """Receive operation details per Cloud Event.

    All required parameters must be populated in order to send to Azure.

    :ivar broker_properties: The Event Broker details. Required.
    :vartype broker_properties: ~azure.eventgrid.models.BrokerProperties
    :ivar event: Cloud Event details. Required.
    :vartype event: ~azure.core.messaging.CloudEvent
    """

    @overload
    def __init__(
        self,
        *,
        broker_properties: "BrokerProperties",
        event: CloudEvent[DataType],
    ): ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:  # pylint: disable=useless-super-delegation
        super().__init__(*args, **kwargs)


class BrokerProperties(InternalBrokerProperties):
    """Properties of the Event Broker operation.

    All required parameters must be populated in order to send to Azure.

    :ivar lock_token: The token used to lock the event. Required.
    :vartype lock_token: str
    :ivar delivery_count: The attempt count for deliverying the event. Required.
    :vartype delivery_count: int
    """

    @overload
    def __init__(
        self,
        *,
        lock_token: str,
        delivery_count: int,
    ): ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]):
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:  # pylint: disable=useless-super-delegation
        super().__init__(*args, **kwargs)


__all__: List[str] = [
    "ReceiveDetails",
    "BrokerProperties",
]  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
FILE: sample_publish_cncf_events_async.py
DESCRIPTION:
    This sample demonstrates creating sending a cloud event from the CNCF library.
USAGE:
    python sample_publish_cncf_events_async.py
    Set the environment variables with your own values before running the sample:
    1) EVENTGRID_KEY - The access key of your eventgrid account.
    2) EVENTGRID_ENDPOINT - The namespace hostname. Typically it exists in the format
    "https://<YOUR-TOPIC-NAME>.<REGION-NAME>.eventgrid.azure.net".
    3) EVENTGRID_TOPIC_NAME - The namespace topic name.
"""
import os
import asyncio
from azure.eventgrid.aio import EventGridPublisherClient
from azure.core.credentials import AzureKeyCredential
from cloudevents.http import CloudEvent

topic_key = os.environ["EVENTGRID_KEY"]
endpoint = os.environ["EVENTGRID_ENDPOINT"]
topic_name = os.environ["EVENTGRID_TOPIC_NAME"]

credential = AzureKeyCredential(topic_key)
client = EventGridPublisherClient(endpoint, credential, namespace_topic=topic_name)


async def publish():
    async with client:
        await client.send(
            [
                CloudEvent(
                    attributes={
                        "type": "cloudevent",
                        "source": "/cncf/cloud/event/1.0",
                        "subject": "testing-cncf-event",
                    },
                    data=b"This is a cncf cloud event.",
                )
            ]
        )


if __name__ == "__main__":
    asyncio.run(publish())



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import sys
import pytest
import base64

from azure.core.messaging import CloudEvent
from azure.eventgrid._legacy._helpers import _cloud_event_to_generated
from azure.eventgrid import SystemEventNames, EventGridEvent
from _mocks import (
    cloud_storage_dict,
    cloud_storage_string,
    cloud_storage_bytes,
)


class EventGridSerializationTests:

    def _assert_cloud_event_serialized(self, expected, actual):
        assert expected["source"] == actual["source"]
        assert expected["type"] == actual["type"]
        assert actual["specversion"] == "1.0"
        assert "id" in actual
        assert "time" in actual

    # Cloud Event tests
    def test_cloud_event_serialization_extension_bytes(self, **kwargs):
        data = b"cloudevent"
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data=data,
            type="Sample.Cloud.Event",
            extensions={"e1": 1, "e2": 2},
        )

        cloud_event.subject = "subject"  # to test explicit setting of prop
        encoded = base64.b64encode(data).decode("utf-8")
        internal = _cloud_event_to_generated(cloud_event)

        assert internal.additional_properties is not None
        assert "e1" in internal.additional_properties

        json = internal.serialize()

        expected = {
            "source": "http://samplesource.dev",
            "data_base64": encoded,
            "type": "Sample.Cloud.Event",
            "reason_code": 204,
            "e1": 1,
            "e2": 2,
        }

        self._assert_cloud_event_serialized(expected, json)
        assert expected["data_base64"] == json["data_base64"]

    def test_cloud_event_serialization_extension_string(self, **kwargs):
        data = "cloudevent"
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data=data,
            type="Sample.Cloud.Event",
            extensions={"e1": 1, "e2": 2},
        )

        cloud_event.subject = "subject"  # to test explicit setting of prop
        internal = _cloud_event_to_generated(cloud_event)

        assert internal.additional_properties is not None
        assert "e1" in internal.additional_properties

        json = internal.serialize()

        expected = {
            "source": "http://samplesource.dev",
            "data": data,
            "type": "Sample.Cloud.Event",
            "reason_code": 204,
            "e1": 1,
            "e2": 2,
        }

        self._assert_cloud_event_serialized(expected, json)
        if sys.version_info > (3, 5):
            assert expected["data"] == json["data"]
        else:
            encoded = base64.b64encode(data).decode("utf-8")
            expected["data_base64"] = encoded
            assert expected["data_base64"] == json["data_base64"]
            assert "data" not in json

    def test_event_grid_event_raises_on_no_data(self):
        with pytest.raises(TypeError):
            eg_event = EventGridEvent(
                subject="sample",
                event_type="Sample.EventGrid.Event",
                data_version="2.0",
            )

    def test_import_from_system_events(self):
        var = SystemEventNames.AcsChatMemberAddedToThreadWithUserEventName
        assert var == "Microsoft.Communication.ChatMemberAddedToThreadWithUser"
        assert SystemEventNames.KeyVaultKeyNearExpiryEventName == "Microsoft.KeyVault.KeyNearExpiry"
        var = SystemEventNames.ServiceBusActiveMessagesAvailableWithNoListenersEventName
        assert var == "Microsoft.ServiceBus.ActiveMessagesAvailableWithNoListeners"
        var = SystemEventNames.AcsChatThreadParticipantAddedEventName
        assert var == "Microsoft.Communication.ChatThreadParticipantAdded"
        var = SystemEventNames.AcsChatThreadParticipantRemovedEventName
        assert var == "Microsoft.Communication.ChatThreadParticipantRemoved"

    def test_eg_event_repr(self):
        event = EventGridEvent(
            subject="sample2",
            data="eventgridevent2",
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )

        assert "EventGridEvent(subject=sample2" in event.__repr__()

    def test_servicebus_system_events_alias(self):
        val = "Microsoft.ServiceBus.DeadletterMessagesAvailableWithNoListeners"
        assert (
            SystemEventNames.ServiceBusDeadletterMessagesAvailableWithNoListenerEventName
            == SystemEventNames.ServiceBusDeadletterMessagesAvailableWithNoListenersEventName
        )
        assert SystemEventNames.ServiceBusDeadletterMessagesAvailableWithNoListenerEventName == val
        assert SystemEventNames.ServiceBusDeadletterMessagesAvailableWithNoListenersEventName == val
        assert SystemEventNames(val) == SystemEventNames.ServiceBusDeadletterMessagesAvailableWithNoListenerEventName
        assert SystemEventNames(val) == SystemEventNames.ServiceBusDeadletterMessagesAvailableWithNoListenersEventName
        with pytest.raises(ValueError):
            SystemEventNames("Microsoft.ServiceBus.DeadletterMessagesAvailableWithNoListener")



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
FILE: sample_consume_process_events.py
DESCRIPTION:
    These samples demonstrate sending, receiving, releasing, and acknowledging CloudEvents.
USAGE:
    python sample_consume_process_events.py
    Set the environment variables with your own values before running the sample:
    1) EVENTGRID_KEY - The access key of your eventgrid account.
    2) EVENTGRID_ENDPOINT - The namespace endpoint. Typically it exists in the format
    "https://<YOUR-NAMESPACE-NAME>.<REGION-NAME>.eventgrid.azure.net".
    3) EVENTGRID_TOPIC_NAME - The namespace topic name.
    4) EVENTGRID_EVENT_SUBSCRIPTION_NAME - The event subscription name.
"""
import os
from azure.core.credentials import AzureKeyCredential
from azure.eventgrid.models import *
from azure.core.messaging import CloudEvent
from azure.core.exceptions import HttpResponseError
from azure.eventgrid import EventGridConsumerClient, EventGridPublisherClient

EVENTGRID_KEY: str = os.environ["EVENTGRID_KEY"]
EVENTGRID_ENDPOINT: str = os.environ["EVENTGRID_ENDPOINT"]
TOPIC_NAME: str = os.environ["EVENTGRID_TOPIC_NAME"]
EVENT_SUBSCRIPTION_NAME: str = os.environ["EVENTGRID_EVENT_SUBSCRIPTION_NAME"]


# Create a client
publisher = EventGridPublisherClient(EVENTGRID_ENDPOINT, AzureKeyCredential(EVENTGRID_KEY), namespace_topic=TOPIC_NAME)
client = EventGridConsumerClient(
    EVENTGRID_ENDPOINT,
    AzureKeyCredential(EVENTGRID_KEY),
    namespace_topic=TOPIC_NAME,
    subscription=EVENT_SUBSCRIPTION_NAME,
)


cloud_event_reject = CloudEvent(data="reject", source="https://example.com", type="example")
cloud_event_release = CloudEvent(data="release", source="https://example.com", type="example")
cloud_event_ack = CloudEvent(data="acknowledge", source="https://example.com", type="example")
cloud_event_renew = CloudEvent(data="renew", source="https://example.com", type="example")

# Send Cloud Events
publisher.send(
    [
        cloud_event_reject,
        cloud_event_release,
        cloud_event_ack,
        cloud_event_renew,
    ]
)

# Receive Published Cloud Events
try:
    receive_results = client.receive(
        max_events=10,
        max_wait_time=10,
    )
except HttpResponseError:
    raise

# Iterate through the results and collect the lock tokens for events we want to release/acknowledge/reject/renew:

release_events = []
acknowledge_events = []
reject_events = []
renew_events = []

for detail in receive_results:
    data = detail.event.data
    broker_properties = detail.broker_properties
    if data == "release":
        release_events.append(broker_properties.lock_token)
    elif data == "acknowledge":
        acknowledge_events.append(broker_properties.lock_token)
    elif data == "renew":
        renew_events.append(broker_properties.lock_token)
    else:
        reject_events.append(broker_properties.lock_token)

# Release/Acknowledge/Reject/Renew events

if len(release_events) > 0:
    try:
        release_result = client.release(
            lock_tokens=release_events,
        )
    except HttpResponseError:
        raise

    for succeeded_lock_token in release_result.succeeded_lock_tokens:
        print(f"Succeeded Lock Token:{succeeded_lock_token}")

if len(acknowledge_events) > 0:
    try:
        ack_result = client.acknowledge(
            lock_tokens=acknowledge_events,
        )
    except HttpResponseError:
        raise

    for succeeded_lock_token in ack_result.succeeded_lock_tokens:
        print(f"Succeeded Lock Token:{succeeded_lock_token}")

if len(reject_events) > 0:
    try:
        reject_result = client.reject(
            lock_tokens=reject_events,
        )
    except HttpResponseError:
        raise

    for succeeded_lock_token in reject_result.succeeded_lock_tokens:
        print(f"Succeeded Lock Token:{succeeded_lock_token}")

if len(renew_events) > 0:
    try:
        renew_result = client.renew_locks(
            lock_tokens=renew_events,
        )
    except HttpResponseError:
        raise

    for succeeded_lock_token in renew_result.succeeded_lock_tokens:
        print(f"Succeeded Lock Token:{succeeded_lock_token}")



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import json
import sys
from typing import Any, Callable, Dict, IO, List, Optional, TypeVar, Union, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from .. import models as _models
from .._model_base import SdkJSONEncoder, _deserialize
from .._serialization import Serializer
from .._validation import api_version_validation
from .._vendor import EventGridConsumerClientMixinABC, EventGridPublisherClientMixinABC

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object
_Unset: Any = object()

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_event_grid_publisher_send_request(topic_name: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: str = kwargs.pop("content_type")
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2024-06-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/topics/{topicName}:publish"
    path_format_arguments = {
        "topicName": _SERIALIZER.url("topic_name", topic_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["content-type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_event_grid_publisher_send_events_request(  # pylint: disable=name-too-long
    topic_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: str = kwargs.pop("content_type")
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2024-06-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/topics/{topicName}:publish"
    path_format_arguments = {
        "topicName": _SERIALIZER.url("topic_name", topic_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["content-type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_event_grid_consumer_receive_request(  # pylint: disable=name-too-long
    topic_name: str,
    event_subscription_name: str,
    *,
    max_events: Optional[int] = None,
    max_wait_time: Optional[int] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2024-06-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/topics/{topicName}/eventsubscriptions/{eventSubscriptionName}:receive"
    path_format_arguments = {
        "topicName": _SERIALIZER.url("topic_name", topic_name, "str"),
        "eventSubscriptionName": _SERIALIZER.url("event_subscription_name", event_subscription_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    if max_events is not None:
        _params["maxEvents"] = _SERIALIZER.query("max_events", max_events, "int")
    if max_wait_time is not None:
        _params["maxWaitTime"] = _SERIALIZER.query("max_wait_time", max_wait_time, "int")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_event_grid_consumer_acknowledge_request(  # pylint: disable=name-too-long
    topic_name: str, event_subscription_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2024-06-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/topics/{topicName}/eventsubscriptions/{eventSubscriptionName}:acknowledge"
    path_format_arguments = {
        "topicName": _SERIALIZER.url("topic_name", topic_name, "str"),
        "eventSubscriptionName": _SERIALIZER.url("event_subscription_name", event_subscription_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_event_grid_consumer_release_request(  # pylint: disable=name-too-long
    topic_name: str,
    event_subscription_name: str,
    *,
    release_delay_in_seconds: Optional[Union[str, _models.ReleaseDelay]] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2024-06-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/topics/{topicName}/eventsubscriptions/{eventSubscriptionName}:release"
    path_format_arguments = {
        "topicName": _SERIALIZER.url("topic_name", topic_name, "str"),
        "eventSubscriptionName": _SERIALIZER.url("event_subscription_name", event_subscription_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    if release_delay_in_seconds is not None:
        _params["releaseDelayInSeconds"] = _SERIALIZER.query(
            "release_delay_in_seconds", release_delay_in_seconds, "str"
        )

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_event_grid_consumer_reject_request(
    topic_name: str, event_subscription_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2024-06-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/topics/{topicName}/eventsubscriptions/{eventSubscriptionName}:reject"
    path_format_arguments = {
        "topicName": _SERIALIZER.url("topic_name", topic_name, "str"),
        "eventSubscriptionName": _SERIALIZER.url("event_subscription_name", event_subscription_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_event_grid_consumer_renew_locks_request(  # pylint: disable=name-too-long
    topic_name: str, event_subscription_name: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2024-06-01"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/topics/{topicName}/eventsubscriptions/{eventSubscriptionName}:renewLock"
    path_format_arguments = {
        "topicName": _SERIALIZER.url("topic_name", topic_name, "str"),
        "eventSubscriptionName": _SERIALIZER.url("event_subscription_name", event_subscription_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


class EventGridPublisherClientOperationsMixin(EventGridPublisherClientMixinABC):

    @distributed_trace
    def _send(self, topic_name: str, event: _models._models.CloudEvent, **kwargs: Any) -> _models._models.PublishResult:
        """Publish a single Cloud Event to a namespace topic.

        :param topic_name: Topic Name. Required.
        :type topic_name: str
        :param event: Single Cloud Event being published. Required.
        :type event: ~azure.eventgrid.models._models.CloudEvent
        :return: PublishResult. The PublishResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models._models.PublishResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop(
            "content_type", _headers.pop("content-type", "application/cloudevents+json; charset=utf-8")
        )
        cls: ClsType[_models._models.PublishResult] = kwargs.pop("cls", None)

        _content = json.dumps(event, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_event_grid_publisher_send_request(
            topic_name=topic_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(
                _models._models.PublishResult, response.json()  # pylint: disable=protected-access
            )

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def _send_events(
        self, topic_name: str, events: List[_models._models.CloudEvent], **kwargs: Any
    ) -> _models._models.PublishResult:
        """Publish a batch of Cloud Events to a namespace topic.

        :param topic_name: Topic Name. Required.
        :type topic_name: str
        :param events: Array of Cloud Events being published. Required.
        :type events: list[~azure.eventgrid.models._models.CloudEvent]
        :return: PublishResult. The PublishResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models._models.PublishResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop(
            "content_type", _headers.pop("content-type", "application/cloudevents-batch+json; charset=utf-8")
        )
        cls: ClsType[_models._models.PublishResult] = kwargs.pop("cls", None)

        _content = json.dumps(events, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_event_grid_publisher_send_events_request(
            topic_name=topic_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(
                _models._models.PublishResult, response.json()  # pylint: disable=protected-access
            )

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore


class EventGridConsumerClientOperationsMixin(EventGridConsumerClientMixinABC):

    @distributed_trace
    def _receive(
        self,
        topic_name: str,
        event_subscription_name: str,
        *,
        max_events: Optional[int] = None,
        max_wait_time: Optional[int] = None,
        **kwargs: Any
    ) -> _models._models.ReceiveResult:
        """Receive a batch of Cloud Events from a subscription.

        :param topic_name: Topic Name. Required.
        :type topic_name: str
        :param event_subscription_name: Event Subscription Name. Required.
        :type event_subscription_name: str
        :keyword max_events: Max Events count to be received. Minimum value is 1, while maximum value
         is 100 events. If not specified, the default value is 1. Default value is None.
        :paramtype max_events: int
        :keyword max_wait_time: Max wait time value for receive operation in Seconds. It is the time in
         seconds that the server approximately waits for the availability of an event and responds to
         the request. If an event is available, the broker responds immediately to the client. Minimum
         value is 10 seconds, while maximum value is 120 seconds. If not specified, the default value is
         60 seconds. Default value is None.
        :paramtype max_wait_time: int
        :return: ReceiveResult. The ReceiveResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models._models.ReceiveResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models._models.ReceiveResult] = kwargs.pop("cls", None)

        _request = build_event_grid_consumer_receive_request(
            topic_name=topic_name,
            event_subscription_name=event_subscription_name,
            max_events=max_events,
            max_wait_time=max_wait_time,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(
                _models._models.ReceiveResult, response.json()  # pylint: disable=protected-access
            )

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def _acknowledge(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: JSON,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.AcknowledgeResult: ...
    @overload
    def _acknowledge(
        self,
        topic_name: str,
        event_subscription_name: str,
        *,
        lock_tokens: List[str],
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.AcknowledgeResult: ...
    @overload
    def _acknowledge(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.AcknowledgeResult: ...

    @distributed_trace
    def _acknowledge(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        lock_tokens: List[str] = _Unset,
        **kwargs: Any
    ) -> _models.AcknowledgeResult:
        """Acknowledge a batch of Cloud Events. The response will include the set of successfully
        acknowledged lock tokens, along with other failed lock tokens with their corresponding error
        information. Successfully acknowledged events will no longer be available to be received by any
        consumer.

        :param topic_name: Topic Name. Required.
        :type topic_name: str
        :param event_subscription_name: Event Subscription Name. Required.
        :type event_subscription_name: str
        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword lock_tokens: Array of lock tokens. Required.
        :paramtype lock_tokens: list[str]
        :return: AcknowledgeResult. The AcknowledgeResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models.AcknowledgeResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.AcknowledgeResult] = kwargs.pop("cls", None)

        if body is _Unset:
            if lock_tokens is _Unset:
                raise TypeError("missing required argument: lock_tokens")
            body = {"lockTokens": lock_tokens}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_event_grid_consumer_acknowledge_request(
            topic_name=topic_name,
            event_subscription_name=event_subscription_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.AcknowledgeResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def _release(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: JSON,
        *,
        release_delay_in_seconds: Optional[Union[str, _models.ReleaseDelay]] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ReleaseResult: ...
    @overload
    def _release(
        self,
        topic_name: str,
        event_subscription_name: str,
        *,
        lock_tokens: List[str],
        release_delay_in_seconds: Optional[Union[str, _models.ReleaseDelay]] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ReleaseResult: ...
    @overload
    def _release(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: IO[bytes],
        *,
        release_delay_in_seconds: Optional[Union[str, _models.ReleaseDelay]] = None,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ReleaseResult: ...

    @distributed_trace
    @api_version_validation(
        params_added_on={"2023-10-01-preview": ["release_delay_in_seconds"]},
    )
    def _release(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        lock_tokens: List[str] = _Unset,
        release_delay_in_seconds: Optional[Union[str, _models.ReleaseDelay]] = None,
        **kwargs: Any
    ) -> _models.ReleaseResult:
        """Release a batch of Cloud Events. The response will include the set of successfully released
        lock tokens, along with other failed lock tokens with their corresponding error information.
        Successfully released events can be received by consumers.

        :param topic_name: Topic Name. Required.
        :type topic_name: str
        :param event_subscription_name: Event Subscription Name. Required.
        :type event_subscription_name: str
        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword lock_tokens: Array of lock tokens. Required.
        :paramtype lock_tokens: list[str]
        :keyword release_delay_in_seconds: Release cloud events with the specified delay in seconds.
         Known values are: "0", "10", "60", "600", and "3600". Default value is None.
        :paramtype release_delay_in_seconds: str or ~azure.eventgrid.models.ReleaseDelay
        :return: ReleaseResult. The ReleaseResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models.ReleaseResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ReleaseResult] = kwargs.pop("cls", None)

        if body is _Unset:
            if lock_tokens is _Unset:
                raise TypeError("missing required argument: lock_tokens")
            body = {"lockTokens": lock_tokens}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_event_grid_consumer_release_request(
            topic_name=topic_name,
            event_subscription_name=event_subscription_name,
            release_delay_in_seconds=release_delay_in_seconds,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.ReleaseResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def _reject(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: JSON,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RejectResult: ...
    @overload
    def _reject(
        self,
        topic_name: str,
        event_subscription_name: str,
        *,
        lock_tokens: List[str],
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RejectResult: ...
    @overload
    def _reject(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RejectResult: ...

    @distributed_trace
    def _reject(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        lock_tokens: List[str] = _Unset,
        **kwargs: Any
    ) -> _models.RejectResult:
        """Reject a batch of Cloud Events. The response will include the set of successfully rejected lock
        tokens, along with other failed lock tokens with their corresponding error information.
        Successfully rejected events will be dead-lettered and can no longer be received by a consumer.

        :param topic_name: Topic Name. Required.
        :type topic_name: str
        :param event_subscription_name: Event Subscription Name. Required.
        :type event_subscription_name: str
        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword lock_tokens: Array of lock tokens. Required.
        :paramtype lock_tokens: list[str]
        :return: RejectResult. The RejectResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models.RejectResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.RejectResult] = kwargs.pop("cls", None)

        if body is _Unset:
            if lock_tokens is _Unset:
                raise TypeError("missing required argument: lock_tokens")
            body = {"lockTokens": lock_tokens}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_event_grid_consumer_reject_request(
            topic_name=topic_name,
            event_subscription_name=event_subscription_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.RejectResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def _renew_locks(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: JSON,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RenewLocksResult: ...
    @overload
    def _renew_locks(
        self,
        topic_name: str,
        event_subscription_name: str,
        *,
        lock_tokens: List[str],
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RenewLocksResult: ...
    @overload
    def _renew_locks(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.RenewLocksResult: ...

    @distributed_trace
    @api_version_validation(
        method_added_on="2023-10-01-preview",
        params_added_on={
            "2023-10-01-preview": ["api_version", "topic_name", "event_subscription_name", "content_type", "accept"]
        },
    )
    def _renew_locks(
        self,
        topic_name: str,
        event_subscription_name: str,
        body: Union[JSON, IO[bytes]] = _Unset,
        *,
        lock_tokens: List[str] = _Unset,
        **kwargs: Any
    ) -> _models.RenewLocksResult:
        """Renew locks for a batch of Cloud Events. The response will include the set of successfully
        renewed lock tokens, along with other failed lock tokens with their corresponding error
        information. Successfully renewed locks will ensure that the associated event is only available
        to the consumer that holds the renewed lock.

        :param topic_name: Topic Name. Required.
        :type topic_name: str
        :param event_subscription_name: Event Subscription Name. Required.
        :type event_subscription_name: str
        :param body: Is either a JSON type or a IO[bytes] type. Required.
        :type body: JSON or IO[bytes]
        :keyword lock_tokens: Array of lock tokens. Required.
        :paramtype lock_tokens: list[str]
        :return: RenewLocksResult. The RenewLocksResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models.RenewLocksResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.RenewLocksResult] = kwargs.pop("cls", None)

        if body is _Unset:
            if lock_tokens is _Unset:
                raise TypeError("missing required argument: lock_tokens")
            body = {"lockTokens": lock_tokens}
            body = {k: v for k, v in body.items() if v is not None}
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_event_grid_consumer_renew_locks_request(
            topic_name=topic_name,
            event_subscription_name=event_subscription_name,
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.RenewLocksResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Customize generated code here.
Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
from typing import List, Union, Any, Optional, Callable, Dict, TypeVar, TYPE_CHECKING
import sys
from azure.core.messaging import CloudEvent
from azure.core.exceptions import (
    HttpResponseError,
    ResourceNotFoundError,
)
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, AsyncHttpResponse
from ...models._patch import ReceiveDetails
from ._operations import (
    EventGridPublisherClientOperationsMixin as PublisherOperationsMixin,
    EventGridConsumerClientOperationsMixin as ConsumerOperationsMixin,
)
from ... import models as _models
from ..._validation import api_version_validation

from ..._operations._patch import (
    _serialize_events,
)

from ..._legacy import EventGridEvent
from ..._legacy._helpers import _is_eventgrid_event_format

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

if TYPE_CHECKING:
    from cloudevents.http.event import CloudEvent as CNCFCloudEvent


class EventGridPublisherClientOperationsMixin(PublisherOperationsMixin):

    @distributed_trace_async
    async def send(
        self,
        events: Union[
            CloudEvent,
            List[CloudEvent],
            Dict[str, Any],
            List[Dict[str, Any]],
            "CNCFCloudEvent",
            List["CNCFCloudEvent"],
            EventGridEvent,
            List[EventGridEvent],
        ],
        *,
        channel_name: Optional[str] = None,
        content_type: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """Send events to the Event Grid Service.

        :param events: The event(s) to send. If sending to an Event Grid Namespace, the dict, list of dicts,
         or event(s) should be in the format of a CloudEvent.
        :type events: CloudEvent or List[CloudEvent] or Dict[str, Any] or List[Dict[str, Any]]
         or CNCFCloudEvent or List[CNCFCloudEvent] or EventGridEvent or List[EventGridEvent]
        :keyword channel_name: The name of the channel to send the event to. Event Grid Basic Resource only.
        :paramtype channel_name: str or None
        :keyword content_type: The content type of the event. If not specified, the default value is
         "application/cloudevents+json; charset=utf-8".
        :paramtype content_type: str

        :return: None
        :rtype: None
        """
        if self._namespace and channel_name:
            raise ValueError("Channel name is not supported for Event Grid Namespaces.")

        # If a cloud event dict, convert to CloudEvent for serializing
        try:
            if isinstance(events, dict):
                events = CloudEvent.from_dict(events)
            if isinstance(events, list) and isinstance(events[0], dict):
                events = [CloudEvent.from_dict(e) for e in events]
        except Exception:  # pylint: disable=broad-except
            pass

        if self._namespace:
            kwargs["content_type"] = (
                content_type if content_type else "application/cloudevents-batch+json; charset=utf-8"
            )
            if not isinstance(events, list):
                events = [events]

            if isinstance(events[0], EventGridEvent) or _is_eventgrid_event_format(events[0]):
                raise TypeError("EventGridEvent is not supported for Event Grid Namespaces.")
            try:
                # Try to send via namespace
                await self._publish(self._namespace, _serialize_events(events), **kwargs)
            except Exception as exception:
                self._http_response_error_handler(exception)
                raise exception
        else:
            kwargs["content_type"] = content_type if content_type else "application/json; charset=utf-8"
            try:
                await self._publish(events, channel_name=channel_name, **kwargs)
            except Exception as exception:
                self._http_response_error_handler(exception)
                raise exception

    def _http_response_error_handler(self, exception):
        if isinstance(exception, HttpResponseError):
            if exception.status_code == 404:
                raise ResourceNotFoundError(
                    "Resource not found. "
                    "For Event Grid Namespaces, please specify the namespace_topic name on the client. "
                    "For Event Grid Basic, do not specify the namespace_topic name."
                ) from exception
            raise exception


class EventGridConsumerClientOperationsMixin(ConsumerOperationsMixin):

    @distributed_trace_async
    async def receive(
        self,
        *,
        max_events: Optional[int] = None,
        max_wait_time: Optional[int] = None,
        **kwargs: Any,
    ) -> List[ReceiveDetails]:
        """Receive Batch of Cloud Events from the Event Subscription.

        :keyword max_events: Max Events count to be received. Minimum value is 1, while maximum value
         is 100 events. The default is None, meaning it will receive one event if available.
        :paramtype max_events: int
        :keyword max_wait_time: Max wait time value for receive operation in Seconds. It is the time in
         seconds that the server approximately waits for the availability of an event and responds to
         the request. If an event is available, the broker responds immediately to the client. Minimum
         value is 10 seconds, while maximum value is 120 seconds. The default value is None, meaning it
         will wait for 60 seconds.
        :paramtype max_wait_time: int
        :return: ReceiveDetails list of received events and their broker properties.
        :rtype: list[~azure.eventgrid.models.ReceiveDetails]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

        detail_items = []
        receive_result = await self._receive(
            self._namespace,
            self._subscription,
            max_events=max_events,
            max_wait_time=max_wait_time,
            **kwargs,
        )
        for detail_item in receive_result.details:
            deserialized_cloud_event = CloudEvent.from_dict(detail_item.event)
            detail_item.event = deserialized_cloud_event
            detail_items.append(
                ReceiveDetails(
                    broker_properties=detail_item.broker_properties,
                    event=detail_item.event,
                )
            )
        return detail_items

    @distributed_trace_async
    async def acknowledge(
        self,
        *,
        lock_tokens: List[str],
        **kwargs: Any,
    ) -> _models.AcknowledgeResult:
        """Acknowledge a batch of Cloud Events. The response will include the set of successfully
        acknowledged lock tokens, along with other failed lock tokens with their corresponding error
        information. Successfully acknowledged events will no longer be available to be received by any
        consumer.

        :keyword lock_tokens: Array of lock tokens of Cloud Events. Required.
        :paramtype lock_tokens: List[str]
        :return: AcknowledgeResult. The AcknowledgeResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models.AcknowledgeResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return await super()._acknowledge(self._namespace, self._subscription, lock_tokens=lock_tokens, **kwargs)

    @distributed_trace_async
    @api_version_validation(
        params_added_on={"2023-10-01-preview": ["release_delay"]},
    )
    async def release(
        self,
        *,
        lock_tokens: List[str],
        release_delay: Optional[Union[int, _models.ReleaseDelay]] = None,
        **kwargs: Any,
    ) -> _models.ReleaseResult:
        """Release a batch of Cloud Events. The response will include the set of successfully released
        lock tokens, along with other failed lock tokens with their corresponding error information.
        Successfully released events can be received by consumers.

        :keyword lock_tokens: Array of lock tokens of Cloud Events. Required.
        :paramtype lock_tokens: List[str]
        :keyword release_delay: Release cloud events with the specified delay in seconds.
         Known values are: 0, 10, 60, 600, and 3600. Default value is None, indicating no delay.
        :paramtype release_delay: int or ~azure.eventgrid.models.ReleaseDelay
        :return: ReleaseResult. The ReleaseResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models.ReleaseResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return await super()._release(
            self._namespace,
            self._subscription,
            lock_tokens=lock_tokens,
            release_delay_in_seconds=release_delay,
            **kwargs,
        )

    @distributed_trace_async
    async def reject(
        self,
        *,
        lock_tokens: List[str],
        **kwargs: Any,
    ) -> _models.RejectResult:
        """Reject a batch of Cloud Events. The response will include the set of successfully rejected lock
        tokens, along with other failed lock tokens with their corresponding error information.
        Successfully rejected events will be dead-lettered and can no longer be received by a consumer.

        :keyword lock_tokens: Array of lock tokens of Cloud Events. Required.
        :paramtype lock_tokens: List[str]
        :return: RejectResult. The RejectResult is compatible with MutableMapping
        :rtype: ~azure.eventgrid.models.RejectResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return await super()._reject(self._namespace, self._subscription, lock_tokens=lock_tokens, **kwargs)

    @distributed_trace_async
    @api_version_validation(
        method_added_on="2023-10-01-preview",
        params_added_on={"2023-10-01-preview": ["api_version", "content_type", "accept"]},
    )
    async def renew_locks(
        self,
        *,
        lock_tokens: List[str],
        **kwargs: Any,
    ) -> _models.RenewLocksResult:
        """Renew lock for batch of Cloud Events. The server responds with an HTTP 200 status code if the
        request is successfully accepted. The response body will include the set of successfully
        renewed lockTokens, along with other failed lockTokens with their corresponding error
        information.

        :keyword lock_tokens: Array of lock tokens of Cloud Events. Required.
        :paramtype lock_tokens: List[str]
        :return: RenewLocksResult. The RenewLocksResult is compatible with
         MutableMapping
        :rtype: ~azure.eventgrid.models.RenewLocksResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return await super()._renew_locks(self._namespace, self._subscription, lock_tokens=lock_tokens, **kwargs)


__all__: List[str] = [
    "EventGridPublisherClientOperationsMixin",
    "EventGridConsumerClientOperationsMixin",
]  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.
    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """



import functools
from devtools_testutils import EnvironmentVariableLoader

from azure.mgmt.eventgrid.models import Topic, InputSchema, JsonInputSchemaMapping, JsonField, JsonFieldWithDefault


EVENTGRID_TOPIC_PARAM = "eventgrid_topic"
EVENTGRID_TOPIC_LOCATION = "westus"
CLOUD_EVENT_SCHEMA = InputSchema.cloud_event_schema_v1_0
CUSTOM_EVENT_SCHEMA = InputSchema.custom_event_schema
ID_JSON_FIELD = JsonField(source_field="customId")
TOPIC_JSON_FIELD = JsonField(source_field="customTopic")
EVENT_TIME_JSON_FIELD = JsonField(source_field="customEventTime")
EVENT_TYPE_JSON_FIELD_WITH_DEFAULT = JsonFieldWithDefault(source_field="customEventType", default_value="")
SUBJECT_JSON_FIELD_WITH_DEFAULT = JsonFieldWithDefault(source_field="customSubject", default_value="")
DATA_VERSION_JSON_FIELD_WITH_DEFAULT = JsonFieldWithDefault(source_field="customDataVersion", default_value="")
CUSTOM_JSON_INPUT_SCHEMA_MAPPING = JsonInputSchemaMapping(
    id=ID_JSON_FIELD,
    topic=TOPIC_JSON_FIELD,
    event_time=EVENT_TIME_JSON_FIELD,
    event_type=EVENT_TYPE_JSON_FIELD_WITH_DEFAULT,
    subject=SUBJECT_JSON_FIELD_WITH_DEFAULT,
    data_version=DATA_VERSION_JSON_FIELD_WITH_DEFAULT,
)

EventGridPreparer = functools.partial(
    EnvironmentVariableLoader,
    "eventgrid",
    eventgrid_topic_endpoint="https://fakeresource.eastus-1.eventgrid.azure.net/api/events",
    eventgrid_topic_key="fakekeyfakekeyfakekeyfakekeyfakekeyfakekeyA=",
    eventgrid_domain_endpoint="https://fakeresource.eastus-1.eventgrid.azure.net/api/events",
    eventgrid_domain_key="fakekeyfakekeyfakekeyfakekeyfakekeyfakekeyA=",
    eventgrid_cloud_event_topic_endpoint="https://fakeresource.eastus-1.eventgrid.azure.net/api/events",
    eventgrid_cloud_event_topic_key="fakekeyfakekeyfakekeyfakekeyfakekeyfakekeyA=",
    eventgrid_cloud_event_domain_endpoint="https://fakeresource.eastus-1.eventgrid.azure.net/api/events",
    eventgrid_cloud_event_domain_key="fakekeyfakekeyfakekeyfakekeyfakekeyfakekeyA=",
    eventgrid_custom_event_topic_endpoint="https://fakeresource.eastus-1.eventgrid.azure.net/api/events",
    eventgrid_custom_event_topic_key="fakekeyfakekeyfakekeyfakekeyfakekeyfakekeyA=",
    eventgrid_partner_channel_name="fakechannel",
    eventgrid_partner_namespace_topic_endpoint="https://fakeresource.eastus-1.eventgrid.azure.net/api/events",
    eventgrid_partner_namespace_topic_key="fakekeyfakekeyfakekeyfakekeyfakekeyfakekeyA=",
    eventgrid_endpoint="https://fakeresource.eastus-1.eventgrid.azure.net/api/events",
    eventgrid_key="fakekeyfakekeyfakekeyfakekeyfakekeyfakekeyA=",
    eventgrid_topic_name="faketopic",
    eventgrid_event_subscription_name="fakesub",
)



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import logging
import sys
import os
import json
import pytest
import uuid
from datetime import datetime, timedelta
from msrest.serialization import UTC
import datetime as dt

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

from devtools_testutils import AzureRecordedTestCase, recorded_by_proxy

from azure.core.credentials import AzureSasCredential, AzureKeyCredential
from azure.core.messaging import CloudEvent
from azure.core.serialization import NULL
from azure.eventgrid import EventGridPublisherClient, EventGridEvent, generate_sas
from azure.eventgrid._legacy._helpers import _cloud_event_to_generated

from eventgrid_preparer import (
    EventGridPreparer,
)


class TestEventGridPublisherClient(AzureRecordedTestCase):
    def create_eg_publisher_client(self, endpoint, topic=None):
        credential = self.get_credential(EventGridPublisherClient)
        client = self.create_client_from_credential(
            EventGridPublisherClient, credential=credential, endpoint=endpoint, namespace_topic=topic
        )
        return client

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_event_grid_event_data_dict(self, eventgrid_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_topic_endpoint)
        eg_event = EventGridEvent(
            subject="sample",
            data={"sample": "eventgridevent"},
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        client.send(eg_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_event_grid_event_fails_without_full_url(self, eventgrid_topic_endpoint):
        credential = self.get_credential(EventGridPublisherClient)
        parsed_url = urlparse(eventgrid_topic_endpoint)
        client = EventGridPublisherClient(parsed_url.netloc, credential)
        eg_event = EventGridEvent(
            subject="sample",
            data={"sample": "eventgridevent"},
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        with pytest.raises(ValueError):
            client.send(eg_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_event_grid_event_data_as_list(self, eventgrid_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_topic_endpoint)
        eg_event1 = EventGridEvent(
            subject="sample",
            data="eventgridevent",
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        eg_event2 = EventGridEvent(
            subject="sample2",
            data="eventgridevent2",
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        client.send([eg_event1, eg_event2])

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_event_grid_event_data_str(self, eventgrid_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_topic_endpoint)
        eg_event = EventGridEvent(
            subject="sample",
            data="eventgridevent",
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        client.send(eg_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_event_grid_event_data_bytes(self, eventgrid_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_topic_endpoint)
        eg_event = EventGridEvent(
            subject="sample",
            data=b"eventgridevent",
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        with pytest.raises(TypeError, match="Data in EventGridEvent cannot be bytes*"):
            client.send(eg_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_event_grid_event_dict_data_bytes(self, eventgrid_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_topic_endpoint)
        eg_event = {
            "subject": "sample",
            "data": b"eventgridevent",
            "eventType": "Sample.EventGrid.Event",
            "dataVersion": "2.0",
            "id": uuid.uuid4(),
            "eventTime": datetime.now(),
        }
        with pytest.raises(TypeError, match="Data in EventGridEvent cannot be bytes*"):
            client.send(eg_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_event_grid_event_dict_data_dict(self, eventgrid_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_topic_endpoint)
        eg_event = {
            "subject": "sample",
            "data": {"key1": "Sample.EventGrid.Event"},
            "eventType": "Sample.EventGrid.Event",
            "dataVersion": "2.0",
            "id": uuid.uuid4(),
            "eventTime": datetime.now(),
        }
        client.send(eg_event)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_send_event_grid_namespace(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        client = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        eg_event = {
            "subject": "sample",
            "data": {"key1": "Sample.EventGrid.Event"},
            "eventType": "Sample.EventGrid.Event",
            "dataVersion": "2.0",
            "id": uuid.uuid4(),
            "eventTime": datetime.now(),
        }
        with pytest.raises(TypeError):
            client.send(eg_event)

    ### CLOUD EVENT TESTS

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_dict(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )
        client.send(cloud_event)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_send_cloud_event_data_dict_namespace(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        client = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )
        client.send(cloud_event)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_send_cloud_event_data_channel_name_namespace(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        client = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )
        with pytest.raises(ValueError):
            client.send(cloud_event, channel_name="testchannel")

    @pytest.mark.skip("https://github.com/Azure/azure-sdk-for-python/issues/16993")
    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_NULL(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(source="http://samplesource.dev", data=NULL, type="Sample.Cloud.Event")

        def callback(request):
            req = json.loads(request.http_request.body)
            assert req[0].get("data") is None

        client.send(cloud_event, raw_request_hook=callback)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_base64_using_data(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data=b"cloudevent",
            type="Sample.Cloud.Event",
        )

        def callback(request):
            req = json.loads(request.http_request.body)
            assert req[0].get("data_base64") is not None
            assert req[0].get("data") is None

        client.send(cloud_event, raw_response_hook=callback)

    def test_send_cloud_event_fails_on_providing_data_and_b64(self):
        with pytest.raises(ValueError, match="Unexpected keyword arguments data_base64.*"):
            cloud_event = CloudEvent(
                source="http://samplesource.dev",
                data_base64=b"cloudevent",
                data="random data",
                type="Sample.Cloud.Event",
            )

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_none(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(source="http://samplesource.dev", data=None, type="Sample.Cloud.Event")
        client.send(cloud_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_str(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data="cloudevent",
            type="Sample.Cloud.Event",
        )
        client.send(cloud_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_bytes(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data=b"cloudevent",
            type="Sample.Cloud.Event",
        )
        client.send(cloud_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_as_list(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data="cloudevent",
            type="Sample.Cloud.Event",
        )
        client.send([cloud_event])

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_data_with_extensions(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data="cloudevent",
            type="Sample.Cloud.Event",
            extensions={"reasoncode": 204, "extension": "hello"},
        )
        client.send([cloud_event])
        internal = _cloud_event_to_generated(cloud_event).serialize()
        assert "reasoncode" in internal
        assert "extension" in internal
        assert internal["reasoncode"] == 204

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_cloud_event_dict(self, eventgrid_cloud_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_cloud_event_topic_endpoint)
        cloud_event1 = {
            "id": "1234",
            "source": "http://samplesource.dev",
            "specversion": "1.0",
            "data": "cloudevent",
            "type": "Sample.Cloud.Event",
        }
        client.send(cloud_event1)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_send_signature_credential(self, **kwargs):
        eventgrid_topic_endpoint = kwargs.pop("eventgrid_topic_endpoint")
        eventgrid_topic_key = kwargs.pop("eventgrid_topic_key")
        expiration_date_utc = dt.datetime.now(UTC()) + timedelta(hours=1)
        signature = generate_sas(eventgrid_topic_endpoint, eventgrid_topic_key, expiration_date_utc)
        credential = AzureSasCredential(signature)
        client = EventGridPublisherClient(eventgrid_topic_endpoint, credential)
        eg_event = EventGridEvent(
            subject="sample",
            data={"sample": "eventgridevent"},
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        client.send(eg_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_NONE_credential(self, eventgrid_topic_endpoint):
        with pytest.raises(ValueError, match="Parameter 'self._credential' must not be None."):
            client = EventGridPublisherClient(eventgrid_topic_endpoint, None)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_custom_schema_event(self, eventgrid_custom_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_custom_event_topic_endpoint)
        custom_event = {
            "customSubject": "sample",
            "customEventType": "sample.event",
            "customDataVersion": "2.0",
            "customId": "1234",
            "customEventTime": dt.datetime.now(UTC()).isoformat(),
            "customData": "sample data",
        }
        client.send(custom_event)

    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_custom_schema_event_as_list(self, eventgrid_custom_event_topic_endpoint):
        client = self.create_eg_publisher_client(eventgrid_custom_event_topic_endpoint)
        custom_event1 = {
            "customSubject": "sample",
            "customEventType": "sample.event",
            "customDataVersion": "2.0",
            "customId": "1234",
            "customEventTime": dt.datetime.now(UTC()).isoformat(),
            "customData": "sample data",
        }
        custom_event2 = {
            "customSubject": "sample2",
            "customEventType": "sample.event",
            "customDataVersion": "2.0",
            "customId": "12345",
            "customEventTime": dt.datetime.now(UTC()).isoformat(),
            "customData": "sample data 2",
        }
        client.send([custom_event1, custom_event2])

    def test_send_throws_with_bad_credential(self):
        bad_credential = "I am a bad credential"
        with pytest.raises(
            ValueError,
            match="The provided credential should be an instance of a TokenCredential, AzureSasCredential or AzureKeyCredential",
        ):
            client = EventGridPublisherClient("eventgrid_endpoint", bad_credential)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_token_credential(self, eventgrid_topic_endpoint):
        credential = self.get_credential(EventGridPublisherClient)
        client = EventGridPublisherClient(eventgrid_topic_endpoint, credential)
        eg_event = EventGridEvent(
            subject="sample",
            data={"sample": "eventgridevent"},
            event_type="Sample.EventGrid.Event",
            data_version="2.0",
        )
        client.send(eg_event)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    @recorded_by_proxy
    def test_send_partner_namespace(self, eventgrid_partner_namespace_topic_endpoint, eventgrid_partner_channel_name):
        client = self.create_eg_publisher_client(eventgrid_partner_namespace_topic_endpoint)
        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data="cloudevent",
            type="Sample.Cloud.Event",
        )

        def callback(request):
            req = request.http_request.headers
            assert req.get("aeg-channel-name") == eventgrid_partner_channel_name

        client.send(cloud_event, channel_name=eventgrid_partner_channel_name, raw_request_hook=callback)



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
FILE: consume_cloud_events_from_eventhub.py
DESCRIPTION:
    These samples demonstrate receiving events from an Event Hub.
USAGE:
    python consume_cloud_events_from_eventhub.py
    Set the environment variables with your own values before running the sample:
    1) EVENT_HUB_CONN_STR: The connection string to the Event hub account
    3) EVENTHUB_NAME: The name of the eventhub account
"""

# Note: This sample would not work on pypy since azure-eventhub
# depends on uamqp which is not pypy compatible.

import os
import json
from azure.core.messaging import CloudEvent
from azure.eventhub import EventHubConsumerClient
from azure.identity import DefaultAzureCredential

EVENTHUB_NAME = os.environ["EVENT_HUB_NAME"]
EVENTHUB_FULLY_QUALIFIED_NAMESPACE = os.environ["EVENT_HUB_HOSTNAME"]


def on_event(partition_context, event):
    dict_event: CloudEvent = CloudEvent.from_json(event)
    print("data: {}\n".format(dict_event.data))


consumer_client = EventHubConsumerClient(
    fully_qualified_namespace=EVENTHUB_FULLY_QUALIFIED_NAMESPACE,
    credential=DefaultAzureCredential(),
    consumer_group="$Default",
    eventhub_name=EVENTHUB_NAME,
)

with consumer_client:
    event_list = consumer_client.receive(
        on_event=on_event, starting_position="-1", prefetch=5  # "-1" is from the beginning of the partition.
    )



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import logging
import sys
import os
import json
import pytest
import uuid
from msrest.serialization import UTC
import datetime as dt

from devtools_testutils import AzureRecordedTestCase
from azure.core.messaging import CloudEvent
from azure.core.exceptions import HttpResponseError
from azure.eventgrid import EventGridConsumerClient, EventGridPublisherClient
from eventgrid_preparer import (
    EventGridPreparer,
)


class TestEventGridDualClient(AzureRecordedTestCase):
    def create_eg_publisher_client(self, endpoint, topic=None):
        credential = self.get_credential(EventGridPublisherClient)
        client = self.create_client_from_credential(
            EventGridPublisherClient, credential=credential, endpoint=endpoint, namespace_topic=topic
        )
        return client

    def create_eg_consumer_client(self, endpoint, topic=None, subscription=None):
        credential = self.get_credential(EventGridConsumerClient)
        client = self.create_client_from_credential(
            EventGridConsumerClient,
            credential=credential,
            endpoint=endpoint,
            namespace_topic=topic,
            subscription=subscription,
        )
        return client

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_eg_dual_client_send_eventgrid_event(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        event_endpoint = kwargs["eventgrid_topic_endpoint"]
        namespace_client = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        basic_client = self.create_eg_publisher_client(event_endpoint)

        event = {
            "id": uuid.uuid4(),
            "data": {
                "team": "azure-sdk",
                "project": "azure-eventgrid",
            },
            "subject": "Door1",
            "eventType": "Azure.SDK.TestEvent",
            "eventTime": dt.datetime.now(UTC()),
            "dataVersion": "2.0",
        }

        basic_client.send(event)

        with pytest.raises(TypeError):
            namespace_client.send(event)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_eg_dual_client_send_custom_event(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        custom_event_endpoint = kwargs["eventgrid_custom_event_topic_endpoint"]
        namespace_client = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        basic_client = self.create_eg_publisher_client(custom_event_endpoint)

        custom_event = {
            "customSubject": "sample",
            "customEventType": "sample.event",
            "customDataVersion": "2.0",
            "customId": "1234",
            "customEventTime": dt.datetime.now(UTC()).isoformat(),
            "customData": "sample data",
        }

        basic_client.send(custom_event)

        with pytest.raises(HttpResponseError):
            namespace_client.send(custom_event)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_eg_dual_client_send_channel_name(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        cloud_event_endpoint = kwargs["eventgrid_partner_namespace_topic_endpoint"]
        channel_name = kwargs["eventgrid_partner_channel_name"]
        namespace_client = self.create_eg_publisher_client(eventgrid_endpoint, eventgrid_topic_name)
        basic_client = self.create_eg_publisher_client(cloud_event_endpoint)

        cloud_event = CloudEvent(
            source="http://samplesource.dev",
            data={"sample": "cloudevent"},
            type="Sample.Cloud.Event",
        )

        basic_client.send(cloud_event, channel_name=channel_name)

        with pytest.raises(ValueError):
            namespace_client.send(cloud_event, channel_name=channel_name)

    @pytest.mark.live_test_only
    @EventGridPreparer()
    def test_eg_dual_client_consumer(self, **kwargs):
        eventgrid_endpoint = kwargs["eventgrid_endpoint"]
        eventgrid_topic_name = kwargs["eventgrid_topic_name"]
        subscription_name = kwargs["eventgrid_event_subscription_name"]
        cloud_event_endpoint = kwargs["eventgrid_cloud_event_topic_endpoint"]
        namespace_client = self.create_eg_consumer_client(eventgrid_endpoint, eventgrid_topic_name, subscription_name)

        namespace_client.receive()

        with pytest.raises(ValueError):
            basic_client = self.create_eg_consumer_client(cloud_event_endpoint)
            basic_client.receive()



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
FILE: publish_event_grid_events_to_custom_topic_sample.py
DESCRIPTION:
    These samples demonstrate creating a list of Eventgrid Events and sending them as a list
    to custom topic.
USAGE:
    python publish_event_grid_events_to_custom_topic_sample.py
    Set the environment variables with your own values before running the sample:
    1) EVENTGRID_TOPIC_KEY - The access key of your eventgrid account.
    2) EVENTGRID_TOPIC_ENDPOINT - The topic hostname. Typically it exists in the format
    "https://<YOUR-TOPIC-NAME>.<REGION-NAME>.eventgrid.azure.net/api/events".
"""
import os
from random import randint, sample
import time

from azure.core.credentials import AzureKeyCredential
from azure.eventgrid import EventGridPublisherClient, EventGridEvent

key = os.environ["EVENTGRID_TOPIC_KEY"]
endpoint = os.environ["EVENTGRID_TOPIC_ENDPOINT"]

# authenticate client
credential = AzureKeyCredential(key)
client = EventGridPublisherClient(endpoint, credential)
services = ["EventGrid", "ServiceBus", "EventHubs", "Storage"]  # possible values for data field


def publish_event():
    # publish events
    for _ in range(3):

        event_list = []  # list of events to publish
        # create events and append to list
        for j in range(randint(1, 3)):
            sample_members = sample(services, k=randint(1, 4))  # select random subset of team members
            event = EventGridEvent(
                subject="Door1", data={"team": sample_members}, event_type="Azure.Sdk.Demo", data_version="2.0"
            )
            event_list.append(event)

        # publish list of events
        client.send(event_list)
        print("Batch of size {} published".format(len(event_list)))
        time.sleep(randint(1, 5))


if __name__ == "__main__":
    publish_event()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, List, Mapping, Optional, TYPE_CHECKING, overload

from azure.core.exceptions import ODataV4Format

from .. import _model_base
from .._model_base import rest_field

if TYPE_CHECKING:
    from .. import models as _models


class AcknowledgeResult(_model_base.Model):
    """The result of the Acknowledge operation.


    :ivar failed_lock_tokens: Array of FailedLockToken for failed cloud events. Each
     FailedLockToken includes the lock token along with the related error information (namely, the
     error code and description). Required.
    :vartype failed_lock_tokens: list[~azure.eventgrid.models.FailedLockToken]
    :ivar succeeded_lock_tokens: Array of lock tokens for the successfully acknowledged cloud
     events. Required.
    :vartype succeeded_lock_tokens: list[str]
    """

    failed_lock_tokens: List["_models.FailedLockToken"] = rest_field(name="failedLockTokens")
    """Array of FailedLockToken for failed cloud events. Each FailedLockToken includes the lock token
     along with the related error information (namely, the error code and description). Required."""
    succeeded_lock_tokens: List[str] = rest_field(name="succeededLockTokens")
    """Array of lock tokens for the successfully acknowledged cloud events. Required."""

    @overload
    def __init__(
        self,
        *,
        failed_lock_tokens: List["_models.FailedLockToken"],
        succeeded_lock_tokens: List[str],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class BrokerProperties(_model_base.Model):
    """Properties of the Event Broker operation.


    :ivar lock_token: The token of the lock on the event. Required.
    :vartype lock_token: str
    :ivar delivery_count: The attempt count for delivering the event. Required.
    :vartype delivery_count: int
    """

    lock_token: str = rest_field(name="lockToken")
    """The token of the lock on the event. Required."""
    delivery_count: int = rest_field(name="deliveryCount")
    """The attempt count for delivering the event. Required."""


class CloudEvent(_model_base.Model):
    """Properties of an event published to an Azure Messaging EventGrid Namespace topic using the
    CloudEvent 1.0 Schema.


    :ivar id: An identifier for the event. The combination of id and source must be unique for each
     distinct event. Required.
    :vartype id: str
    :ivar source: Identifies the context in which an event happened. The combination of id and
     source must be unique for each distinct event. Required.
    :vartype source: str
    :ivar data: Event data specific to the event type.
    :vartype data: any
    :ivar data_base64: Event data specific to the event type, encoded as a base64 string.
    :vartype data_base64: bytes
    :ivar type: Type of event related to the originating occurrence. Required.
    :vartype type: str
    :ivar time: The time (in UTC) the event was generated, in RFC3339 format.
    :vartype time: ~datetime.datetime
    :ivar specversion: The version of the CloudEvents specification which the event uses. Required.
    :vartype specversion: str
    :ivar dataschema: Identifies the schema that data adheres to.
    :vartype dataschema: str
    :ivar datacontenttype: Content type of data value.
    :vartype datacontenttype: str
    :ivar subject: This describes the subject of the event in the context of the event producer
     (identified by source).
    :vartype subject: str
    """

    id: str = rest_field()
    """An identifier for the event. The combination of id and source must be unique for each distinct
     event. Required."""
    source: str = rest_field()
    """Identifies the context in which an event happened. The combination of id and source must be
     unique for each distinct event. Required."""
    data: Optional[Any] = rest_field()
    """Event data specific to the event type."""
    data_base64: Optional[bytes] = rest_field(format="base64")
    """Event data specific to the event type, encoded as a base64 string."""
    type: str = rest_field()
    """Type of event related to the originating occurrence. Required."""
    time: Optional[datetime.datetime] = rest_field(format="rfc3339")
    """The time (in UTC) the event was generated, in RFC3339 format."""
    specversion: str = rest_field()
    """The version of the CloudEvents specification which the event uses. Required."""
    dataschema: Optional[str] = rest_field()
    """Identifies the schema that data adheres to."""
    datacontenttype: Optional[str] = rest_field()
    """Content type of data value."""
    subject: Optional[str] = rest_field()
    """This describes the subject of the event in the context of the event producer (identified by
     source)."""


class FailedLockToken(_model_base.Model):
    """Failed LockToken information.


    :ivar lock_token: The lock token of an entry in the request. Required.
    :vartype lock_token: str
    :ivar error: Error information of the failed operation result for the lock token in the
     request. Required.
    :vartype error: ~azure.core.ODataV4Format
    """

    lock_token: str = rest_field(name="lockToken")
    """The lock token of an entry in the request. Required."""
    error: ODataV4Format = rest_field()
    """Error information of the failed operation result for the lock token in the request. Required."""

    @overload
    def __init__(
        self,
        *,
        lock_token: str,
        error: ODataV4Format,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class PublishResult(_model_base.Model):
    """The result of the Publish operation."""


class ReceiveDetails(_model_base.Model):
    """Receive operation details per Cloud Event.


    :ivar broker_properties: The Event Broker details. Required.
    :vartype broker_properties: ~azure.eventgrid.models._models.BrokerProperties
    :ivar event: Cloud Event details. Required.
    :vartype event: ~azure.eventgrid.models._models.CloudEvent
    """

    broker_properties: "_models._models.BrokerProperties" = rest_field(name="brokerProperties")
    """The Event Broker details. Required."""
    event: "_models._models.CloudEvent" = rest_field()
    """Cloud Event details. Required."""


class ReceiveResult(_model_base.Model):
    """Details of the Receive operation response.


    :ivar details: Array of receive responses, one per cloud event. Required.
    :vartype details: list[~azure.eventgrid.models._models.ReceiveDetails]
    """

    details: List["_models._models.ReceiveDetails"] = rest_field(name="value")
    """Array of receive responses, one per cloud event. Required."""


class RejectResult(_model_base.Model):
    """The result of the Reject operation.


    :ivar failed_lock_tokens: Array of FailedLockToken for failed cloud events. Each
     FailedLockToken includes the lock token along with the related error information (namely, the
     error code and description). Required.
    :vartype failed_lock_tokens: list[~azure.eventgrid.models.FailedLockToken]
    :ivar succeeded_lock_tokens: Array of lock tokens for the successfully rejected cloud events.
     Required.
    :vartype succeeded_lock_tokens: list[str]
    """

    failed_lock_tokens: List["_models.FailedLockToken"] = rest_field(name="failedLockTokens")
    """Array of FailedLockToken for failed cloud events. Each FailedLockToken includes the lock token
     along with the related error information (namely, the error code and description). Required."""
    succeeded_lock_tokens: List[str] = rest_field(name="succeededLockTokens")
    """Array of lock tokens for the successfully rejected cloud events. Required."""

    @overload
    def __init__(
        self,
        *,
        failed_lock_tokens: List["_models.FailedLockToken"],
        succeeded_lock_tokens: List[str],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ReleaseResult(_model_base.Model):
    """The result of the Release operation.


    :ivar failed_lock_tokens: Array of FailedLockToken for failed cloud events. Each
     FailedLockToken includes the lock token along with the related error information (namely, the
     error code and description). Required.
    :vartype failed_lock_tokens: list[~azure.eventgrid.models.FailedLockToken]
    :ivar succeeded_lock_tokens: Array of lock tokens for the successfully released cloud events.
     Required.
    :vartype succeeded_lock_tokens: list[str]
    """

    failed_lock_tokens: List["_models.FailedLockToken"] = rest_field(name="failedLockTokens")
    """Array of FailedLockToken for failed cloud events. Each FailedLockToken includes the lock token
     along with the related error information (namely, the error code and description). Required."""
    succeeded_lock_tokens: List[str] = rest_field(name="succeededLockTokens")
    """Array of lock tokens for the successfully released cloud events. Required."""

    @overload
    def __init__(
        self,
        *,
        failed_lock_tokens: List["_models.FailedLockToken"],
        succeeded_lock_tokens: List[str],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RenewLocksResult(_model_base.Model):
    """The result of the RenewLock operation.


    :ivar failed_lock_tokens: Array of FailedLockToken for failed cloud events. Each
     FailedLockToken includes the lock token along with the related error information (namely, the
     error code and description). Required.
    :vartype failed_lock_tokens: list[~azure.eventgrid.models.FailedLockToken]
    :ivar succeeded_lock_tokens: Array of lock tokens for the successfully renewed locks. Required.
    :vartype succeeded_lock_tokens: list[str]
    """

    failed_lock_tokens: List["_models.FailedLockToken"] = rest_field(name="failedLockTokens")
    """Array of FailedLockToken for failed cloud events. Each FailedLockToken includes the lock token
     along with the related error information (namely, the error code and description). Required."""
    succeeded_lock_tokens: List[str] = rest_field(name="succeededLockTokens")
    """Array of lock tokens for the successfully renewed locks. Required."""

    @overload
    def __init__(
        self,
        *,
        failed_lock_tokens: List["_models.FailedLockToken"],
        succeeded_lock_tokens: List[str],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""Customize generated code here.
Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""

from typing import List, Union, Optional, Any
from azure.core.credentials import AzureKeyCredential, AzureSasCredential, TokenCredential

from ._client import (
    EventGridPublisherClient as InternalEventGridPublisherClient,
    EventGridConsumerClient as InternalEventGridConsumerClient,
)
from ._legacy import (
    EventGridPublisherClient as LegacyEventGridPublisherClient,
    SystemEventNames,
    EventGridEvent,
    generate_sas,
)
from ._serialization import Deserializer, Serializer


DEFAULT_STANDARD_API_VERSION = "2024-06-01"
DEFAULT_BASIC_API_VERSION = "2018-01-01"


class EventGridPublisherClient(InternalEventGridPublisherClient):
    """EventGridPublisherClient.

    Sends events to a basic topic, basic domain, or a namespace topic
    specified during the client initialization.

    A single instance or a list of dictionaries, CloudEvents or EventGridEvents are accepted.
    If a list is provided, the list must contain only one type of event.
    If dictionaries are provided and sending to a namespace topic,
    the dictionary must follow the CloudEvent schema.

    :param endpoint: The endpoint of the Event Grid tier (basic or namespace), e.g.
     namespaceName1.westus-1.eventgrid.azure.net. Required.
    :type endpoint: str
    :param credential: Credential used to authenticate requests to the service. Is either a
     AzureKeyCredential type or a TokenCredential type. Required.
    :type credential: ~azure.core.credentials.AzureKeyCredential or
     ~azure.core.credentials.TokenCredential
    :keyword namespace_topic: The name of the topic to publish events to. Required for Event Grid Namespaces.
     Default value is None, which is used for Event Grid Basic.
    :paramtype namespace_topic: str or None
    :keyword api_version: The API version to use for this operation. Default value for Event Grid Namespace
     is "2024-06-01", default value for Event Grid Basic is "2018-01-01".
     Note that overriding this default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(
        self,
        endpoint: str,
        credential: Union[AzureKeyCredential, AzureSasCredential, "TokenCredential"],
        *,
        namespace_topic: Optional[str] = None,
        api_version: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        self._namespace = namespace_topic
        self._credential = credential

        if not self._namespace:
            self._client = LegacyEventGridPublisherClient(
                endpoint,
                credential,
                api_version=api_version or DEFAULT_BASIC_API_VERSION,
            )  # type:ignore[assignment]
            self._publish = self._client.send  # type:ignore[attr-defined]
        else:
            if isinstance(credential, AzureSasCredential):
                raise TypeError("SAS token authentication is not supported for Event Grid Namespace.")
            super().__init__(
                endpoint=endpoint,
                credential=credential,
                api_version=api_version or DEFAULT_STANDARD_API_VERSION,
                **kwargs,
            )

            self._publish = self._send_events
        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False

    def __repr__(self) -> str:
        return (
            f"<EventGridPublisherClient: namespace_topic={self._namespace}, credential type={type(self._credential)}>"
        )


class EventGridConsumerClient(InternalEventGridConsumerClient):
    """EventGridConsumerClient.

    Consumes and manages events from a namespace topic
    and event subscription specified during the client initialization.

    :param endpoint: The host name of the namespace, e.g.
     namespaceName1.westus-1.eventgrid.azure.net. Required.
    :type endpoint: str
    :param credential: Credential used to authenticate requests to the service. Is either a
     AzureKeyCredential type or a TokenCredential type. Required.
    :type credential: ~azure.core.credentials.AzureKeyCredential or
     ~azure.core.credentials.TokenCredential
    :keyword namespace_topic: The name of the topic to consume events from. Required.
    :paramtype namespace_topic: str
    :keyword subscription: The name of the subscription to consume events from. Required.
    :paramtype subscription: str
    :keyword api_version: The API version to use for this operation. Default value is "2024-06-01".
     Note that overriding this default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(
        self,
        endpoint: str,
        credential: Union[AzureKeyCredential, "TokenCredential"],
        *,
        namespace_topic: str,
        subscription: str,
        api_version: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        self._namespace = namespace_topic
        self._subscription = subscription
        self._credential = credential
        super().__init__(
            endpoint=endpoint, credential=credential, api_version=api_version or DEFAULT_STANDARD_API_VERSION, **kwargs
        )

    def __repr__(self) -> str:
        return f"<EventGridConsumerClient: namespace_topic={self._namespace}, \
            subscription={self._subscription}, credential type={type(self._credential)}>"


def patch_sdk():
    """Do not remove from this file.
    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


__all__: List[str] = [
    "EventGridPublisherClient",
    "EventGridConsumerClient",
    "SystemEventNames",
    "EventGridEvent",
    "generate_sas",
]  # Add all objects you want publicly available to users at this package level
